'2009 IDesign Inc. 
'Questions? Comments? go to 
'http://www.idesign.net


Imports Microsoft.VisualBasic
Imports System
Imports System.ServiceModel
Imports System.ServiceModel.Security
Imports System.Collections.ObjectModel
Imports System.ServiceModel.MsmqIntegration
Imports System.Security.Cryptography.X509Certificates
Imports System.Diagnostics
Imports System.Web.Security
Imports System.Reflection
Imports System.Net.Security
Imports System.ServiceModel.Description
Imports System.ServiceModel.Channels
Imports System.Web.Configuration
Imports System.Configuration

Namespace ServiceModelEx
   Friend Class SecurityBehavior
	   Implements IServiceBehavior
	  Private m_Mode As ServiceSecurity
	  Private m_StoreLocation As StoreLocation
	  Private m_StoreName As StoreName
	  Private m_FindType As X509FindType
	  Private m_SubjectName As String
	  Private m_UseAspNetProviders As Boolean
	  Private m_ApplicationName As String = String.Empty


	  Private privateImpersonateAll As Boolean
	  Public Property ImpersonateAll() As Boolean
		  Get
			  Return privateImpersonateAll
		  End Get
		  Set(ByVal value As Boolean)
			  privateImpersonateAll = value
		  End Set
	  End Property
	  Public Property ApplicationName() As String
		 Get
			Return m_ApplicationName
		 End Get
		 Set(ByVal value As String)
			m_ApplicationName = value
		 End Set
	  End Property
	  Public Property UseAspNetProviders() As Boolean
		 Get
			Return m_UseAspNetProviders
		 End Get
		 Set(ByVal value As Boolean)
			m_UseAspNetProviders = value
			If value = True AndAlso Roles.Enabled = False Then
			   EnableRoleManagerInConfig()
			End If
		 End Set
	  End Property
	  ''' <summary>
	  ''' </summary>
	  ''' <param name="mode">If set to ServiceSecurity.Anonymous,ServiceSecurity.BusinessToBusiness or ServiceSecurity.Internet then the service certificate must be listed in config file</param>
	  Public Sub New(ByVal mode As ServiceSecurity)
		  Me.New(mode,StoreLocation.LocalMachine,StoreName.My,X509FindType.FindBySubjectName,Nothing)
	  End Sub
	  ''' <summary>
	  ''' </summary>
	  ''' <param name="mode">Certificate is looked up by name from LocalMachine/My store</param>
	  Public Sub New(ByVal mode As ServiceSecurity, ByVal serviceCertificateName As String)
		  Me.New(mode,StoreLocation.LocalMachine,StoreName.My,X509FindType.FindBySubjectName,serviceCertificateName)
	  End Sub
	  Public Sub New(ByVal mode As ServiceSecurity, ByVal storeLocation As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType, ByVal subjectName As String)
		 m_Mode = mode
		 m_StoreLocation = storeLocation
		 m_StoreName = storeName
		 m_FindType = findType
		 m_SubjectName = subjectName
	  End Sub

	  Public Sub Validate(ByVal description As ServiceDescription, ByVal serviceHostBase As ServiceHostBase) Implements IServiceBehavior.Validate
		 If m_SubjectName IsNot Nothing Then
			Select Case m_Mode
			   Case ServiceSecurity.Anonymous, ServiceSecurity.BusinessToBusiness, ServiceSecurity.Internet
				  Dim subjectName As String
				  If m_SubjectName <> String.Empty Then
					 subjectName = m_SubjectName
				  Else
					 subjectName = description.Endpoints(0).Address.Uri.Host
				  End If
				  serviceHostBase.Credentials.ServiceCertificate.SetCertificate(m_StoreLocation,m_StoreName,m_FindType,subjectName)
				  Exit Select
			End Select
		 End If
		 If UseAspNetProviders = True Then
			Debug.Assert(serviceHostBase.Credentials IsNot Nothing)
			serviceHostBase.Authorization.PrincipalPermissionMode = PrincipalPermissionMode.UseAspNetRoles
			Dim applicationName As String
			Debug.Assert(Roles.ApplicationName = Membership.ApplicationName)
			If String.IsNullOrEmpty(Me.ApplicationName) Then
			   Me.ApplicationName = Membership.ApplicationName
			End If
			If String.IsNullOrEmpty(Me.ApplicationName) OrElse Me.ApplicationName = "/" Then
			   If String.IsNullOrEmpty(System.Reflection.Assembly.GetEntryAssembly().GetName().Name) Then
				  applicationName = AppDomain.CurrentDomain.FriendlyName
			   Else
				  applicationName = System.Reflection.Assembly.GetEntryAssembly().GetName().Name
			   End If
			Else
			   applicationName = Me.ApplicationName
			End If
			Membership.ApplicationName = applicationName
			Roles.ApplicationName = applicationName

			If m_Mode = ServiceSecurity.Internet Then
			   serviceHostBase.Credentials.UserNameAuthentication.UserNamePasswordValidationMode = UserNamePasswordValidationMode.MembershipProvider
			End If
		 Else
			Debug.Assert(m_ApplicationName Is Nothing)
			'Reiterate the defaults 
			serviceHostBase.Credentials.UserNameAuthentication.UserNamePasswordValidationMode = UserNamePasswordValidationMode.Windows
			serviceHostBase.Authorization.PrincipalPermissionMode = PrincipalPermissionMode.UseWindowsGroups
		 End If
		 If m_Mode = ServiceSecurity.Anonymous OrElse m_Mode = ServiceSecurity.BusinessToBusiness AndAlso UseAspNetProviders = False Then
			serviceHostBase.Authorization.PrincipalPermissionMode = PrincipalPermissionMode.None
		 End If

		 'Make it affect only when possible 
		 If ImpersonateAll Then
			If m_Mode = ServiceSecurity.Intranet OrElse (m_Mode = ServiceSecurity.Internet AndAlso UseAspNetProviders = False) Then
			   Return
			Else
			   ImpersonateAll = False
			End If
		 End If
		 If m_Mode = ServiceSecurity.BusinessToBusiness Then
			serviceHostBase.Credentials.ClientCertificate.Authentication.CertificateValidationMode = X509CertificateValidationMode.PeerTrust
		 End If
	  End Sub

	  Public Sub AddBindingParameters(ByVal description As ServiceDescription, ByVal serviceHostBase As ServiceHostBase, ByVal endpoints As Collection(Of ServiceEndpoint), ByVal parameters As BindingParameterCollection) Implements IServiceBehavior.AddBindingParameters
		 If ImpersonateAll Then
			Debug.Assert(description Is serviceHostBase.Description)
			serviceHostBase.ImpersonateAll()
		 End If
		 Select Case m_Mode
			Case ServiceSecurity.None
			   ConfigureNone(endpoints)
			   Exit Select
			Case ServiceSecurity.Anonymous
			   ConfigureAnonymous(endpoints)
			   Exit Select
			Case ServiceSecurity.BusinessToBusiness
			   ConfigureBusinessToBusiness(endpoints)
			   Exit Select
			Case ServiceSecurity.Internet
			   ConfigureInternet(endpoints,UseAspNetProviders)
			   Exit Select
			Case ServiceSecurity.Intranet
			   ConfigureIntranet(endpoints)
			   Exit Select
			Case Else
			   Throw New InvalidOperationException(m_Mode & " is unrecognized security mode")
		 End Select
	  End Sub
	  Public Sub ApplyDispatchBehavior(ByVal description As ServiceDescription, ByVal serviceHostBase As ServiceHostBase) Implements IServiceBehavior.ApplyDispatchBehavior
	  End Sub
	   Friend Shared Sub ConfigureNone(ByVal endpoints As Collection(Of ServiceEndpoint))
		 For Each endpoint As ServiceEndpoint In endpoints
			Dim binding As Binding = endpoint.Binding

			If TypeOf binding Is BasicHttpBinding Then
			   Dim basicBinding As BasicHttpBinding = CType(binding, BasicHttpBinding)
			   basicBinding.Security.Mode = BasicHttpSecurityMode.None
			   Continue For
			End If
			If TypeOf binding Is NetTcpBinding Then
			   Dim tcpBinding As NetTcpBinding = CType(binding, NetTcpBinding)
			   tcpBinding.Security.Mode = SecurityMode.None
			   Continue For
			End If
			If TypeOf binding Is NetPeerTcpBinding Then
			   Dim peerBinding As NetPeerTcpBinding = CType(binding, NetPeerTcpBinding)
			   peerBinding.Security.Mode = SecurityMode.None
			   Continue For
			End If
			If TypeOf binding Is NetNamedPipeBinding Then
			   Dim pipeBinding As NetNamedPipeBinding = CType(binding, NetNamedPipeBinding)
			   pipeBinding.Security.Mode = NetNamedPipeSecurityMode.None
			   Continue For
			End If
			If TypeOf binding Is WSHttpBinding Then
			   Dim wsBinding As WSHttpBinding = CType(binding, WSHttpBinding)
			   wsBinding.Security.Mode = SecurityMode.None
			   Continue For
			End If
			If TypeOf binding Is WSDualHttpBinding Then
			   Dim wsDualBinding As WSDualHttpBinding = CType(binding, WSDualHttpBinding)
			   wsDualBinding.Security.Mode = WSDualHttpSecurityMode.None
			   Continue For
			End If
			If TypeOf binding Is NetMsmqBinding Then
			   Dim msmqBinding As NetMsmqBinding = CType(binding, NetMsmqBinding)
			   msmqBinding.Security.Mode = NetMsmqSecurityMode.None
			   Continue For
			End If
			If TypeOf binding Is CustomBinding AndAlso endpoint.Contract.ContractType Is GetType(IMetadataExchange) Then
			   Trace.WriteLine("No declarative security for MEX endpoint when adding it programmatically")
			   Continue For
			End If
                Throw New InvalidOperationException(binding.GetType().ToString & "is unsupported with ServiceSecurity.None")
		 Next endpoint
	   End Sub
	  Friend Shared Sub ConfigureAnonymous(ByVal endpoints As Collection(Of ServiceEndpoint))
		 For Each endpoint As ServiceEndpoint In endpoints
			Dim binding As Binding = endpoint.Binding

			If TypeOf binding Is NetTcpBinding Then
			   Dim tcpBinding As NetTcpBinding = CType(binding, NetTcpBinding)
			   tcpBinding.Security.Mode = SecurityMode.Message
			   tcpBinding.Security.Message.ClientCredentialType = MessageCredentialType.None
			   Continue For
			End If
			If TypeOf binding Is WSHttpBinding Then
			   Dim wsBinding As WSHttpBinding = CType(binding, WSHttpBinding)
			   wsBinding.Security.Mode = SecurityMode.Message
			   wsBinding.Security.Message.ClientCredentialType = MessageCredentialType.None
			   Continue For
			End If
			If TypeOf binding Is WSDualHttpBinding Then
			   Dim wsDualBinding As WSDualHttpBinding = CType(binding, WSDualHttpBinding)
			   wsDualBinding.Security.Mode = WSDualHttpSecurityMode.Message
			   wsDualBinding.Security.Message.ClientCredentialType = MessageCredentialType.None
			   Continue For
			End If
			If TypeOf binding Is NetMsmqBinding Then
			   Dim msmqBinding As NetMsmqBinding = CType(binding, NetMsmqBinding)
			   msmqBinding.Security.Mode = NetMsmqSecurityMode.Message
			   msmqBinding.Security.Message.ClientCredentialType = MessageCredentialType.None
			   Continue For
			End If
			If TypeOf binding Is CustomBinding AndAlso endpoint.Contract.ContractType Is GetType(IMetadataExchange) Then
			   Trace.WriteLine("No declarative security for MEX endpoint when adding it programmatically")
			   Continue For
			End If
                Throw New InvalidOperationException(binding.GetType().ToString & "is unsupported with ServiceSecurity.Anonymous")
		 Next endpoint
	  End Sub
	  Friend Shared Sub ConfigureBusinessToBusiness(ByVal endpoints As Collection(Of ServiceEndpoint))
		 For Each endpoint As ServiceEndpoint In endpoints
			Dim binding As Binding = endpoint.Binding

			If TypeOf binding Is BasicHttpBinding Then
			   Dim basicBinding As BasicHttpBinding = CType(binding, BasicHttpBinding)
			   basicBinding.Security.Mode = BasicHttpSecurityMode.Message
			   basicBinding.Security.Message.ClientCredentialType = BasicHttpMessageCredentialType.Certificate
			   Continue For
			End If
			If TypeOf binding Is WSHttpBinding Then
			   Dim wsBinding As WSHttpBinding = CType(binding, WSHttpBinding)
			   wsBinding.Security.Mode = SecurityMode.Message
			   wsBinding.Security.Message.ClientCredentialType = MessageCredentialType.Certificate
			   Continue For
			End If
			If TypeOf binding Is WSDualHttpBinding Then
			   Dim wsDualBinding As WSDualHttpBinding = CType(binding, WSDualHttpBinding)
			   wsDualBinding.Security.Mode = WSDualHttpSecurityMode.Message
			   wsDualBinding.Security.Message.ClientCredentialType = MessageCredentialType.Certificate
			   Continue For
			End If
			If TypeOf binding Is CustomBinding AndAlso endpoint.Contract.ContractType Is GetType(IMetadataExchange) Then
			   Trace.WriteLine("No declarative security for MEX endpoint when adding it programmatically")
			   Continue For
			End If
                Throw New InvalidOperationException(binding.GetType().ToString & "is unsupported with ServiceSecurity.BusinessToBusiness")
		 Next endpoint
	  End Sub
	  Friend Shared Sub ConfigureInternet(ByVal endpoints As Collection(Of ServiceEndpoint), ByVal useAspNetProviders As Boolean)
		 For Each endpoint As ServiceEndpoint In endpoints
			Dim binding As Binding = endpoint.Binding

			If TypeOf binding Is WSHttpBinding Then
			   Dim wsBinding As WSHttpBinding = CType(binding, WSHttpBinding)
			   wsBinding.Security.Mode = SecurityMode.Message
			   wsBinding.Security.Message.ClientCredentialType = MessageCredentialType.UserName
			   Continue For
			End If
			If TypeOf binding Is WSDualHttpBinding Then
			   Dim wsDualBinding As WSDualHttpBinding = CType(binding, WSDualHttpBinding)
			   wsDualBinding.Security.Mode = WSDualHttpSecurityMode.Message
			   wsDualBinding.Security.Message.ClientCredentialType = MessageCredentialType.UserName
			   Continue For
			End If
			If TypeOf binding Is NetTcpBinding Then
			   Debug.Assert(useAspNetProviders = True,"You should be using Windows security")
			   Dim tcpBinding As NetTcpBinding = CType(binding, NetTcpBinding)
			   tcpBinding.Security.Mode = SecurityMode.Message
			   tcpBinding.Security.Message.ClientCredentialType = MessageCredentialType.UserName
			   Continue For
			End If
			If TypeOf binding Is CustomBinding AndAlso endpoint.Contract.ContractType Is GetType(IMetadataExchange) Then
			   Trace.WriteLine("No declarative security for MEX endpoint when adding it programmatically")
			   Continue For
			End If
                Throw New InvalidOperationException(binding.GetType().ToString & "is unsupported with ServiceSecurity.Internet")
		 Next endpoint
	  End Sub
	  Friend Shared Sub ConfigureIntranet(ByVal endpoints As Collection(Of ServiceEndpoint))
		 For Each endpoint As ServiceEndpoint In endpoints
			Dim binding As Binding = endpoint.Binding

			If TypeOf binding Is NetTcpBinding Then
			   Dim tcpBinding As NetTcpBinding = CType(binding, NetTcpBinding)
			   tcpBinding.Security.Mode = SecurityMode.Transport
			   tcpBinding.Security.Transport.ClientCredentialType = TcpClientCredentialType.Windows

			   tcpBinding.Security.Transport.ProtectionLevel = ProtectionLevel.EncryptAndSign
			   Continue For
			End If
			If TypeOf binding Is NetNamedPipeBinding Then
			   Dim pipeBinding As NetNamedPipeBinding = CType(binding, NetNamedPipeBinding)
			   pipeBinding.Security.Mode = NetNamedPipeSecurityMode.Transport
			   pipeBinding.Security.Transport.ProtectionLevel = ProtectionLevel.EncryptAndSign
			   Continue For
			End If
			If TypeOf binding Is NetMsmqBinding Then
			   Dim msmqBinding As NetMsmqBinding = CType(binding, NetMsmqBinding)
			   msmqBinding.Security.Mode = NetMsmqSecurityMode.Transport
			   msmqBinding.Security.Transport.MsmqAuthenticationMode = MsmqAuthenticationMode.WindowsDomain
			   msmqBinding.Security.Transport.MsmqProtectionLevel = ProtectionLevel.EncryptAndSign
			   Continue For
			End If
			If TypeOf binding Is CustomBinding AndAlso endpoint.Contract.ContractType Is GetType(IMetadataExchange) Then
			   Trace.WriteLine("No declarative security for MEX endpoint when adding it programmatically")
			   Continue For
			End If
                Throw New InvalidOperationException(binding.GetType().ToString & "is unsupported with ServiceSecurity.Intranet")
		 Next endpoint
	  End Sub
	  Friend Shared Sub EnableRoleManagerInConfig()
		 Try
			'Get the role manager section.
			Dim roleManagerSection As RoleManagerSection = CType(ConfigurationManager.GetSection("system.web/roleManager"), RoleManagerSection)

			'Use reflection to enable changing it
			Dim fieldInfo As FieldInfo = GetType(ConfigurationElement).GetField("_bReadOnly",BindingFlags.Instance Or BindingFlags.NonPublic)
			fieldInfo.SetValue(roleManagerSection,False)

			roleManagerSection.Enabled = True
		 Catch
		 End Try
	  End Sub
   End Class
End Namespace