'2009 IDesign Inc. 
'Questions? Comments? go to 
'http://www.idesign.net



Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Reflection
Imports System.Diagnostics
Imports System.ServiceModel
Imports Microsoft.ServiceBus
Imports System.Runtime.CompilerServices
Imports System.Security.Cryptography.X509Certificates

Namespace ServiceModelEx
    Public Class EventRelayHost
        'For serivce cert lookup
        Private m_ServiceCertLocation As StoreLocation
        Private m_ServiceCertStoreName As StoreName
        Private m_ServiceCertFindType As X509FindType
        Private m_ServiceCertFindValue As Object

        'For message security
        Private m_ApplicationName As String
        Private m_UseProviders As Boolean
        Private m_Anonymous As Boolean

        'Managing the host-per-event
        Private m_Hosts As Dictionary(Of Type, Dictionary(Of String, ServiceBusHost)) = New Dictionary(Of Type, Dictionary(Of String, ServiceBusHost))()
        Private m_SericeType As Type
        Private m_BaseAddresses() As String
        Private m_SingletonInstance As Object
        Private m_Binding As NetEventRelayBinding

        'Service bus credentials
        Private m_ServiceBusPassword As String
        Private m_ServiceBusCertLocation As StoreLocation
        Private m_ServiceBusCertStoreName As StoreName
        Private m_ServiceBusCertFindType As X509FindType
        Private m_ServiceBusCertFindValue As Object

        Public Sub New(ByVal singletonInstance As Object, ByVal baseAddress As String)
            Me.New(singletonInstance, New String() {baseAddress})
        End Sub
        Public Sub New(ByVal singletonInstance As Object, ByVal baseAddresses() As String)
            Debug.Assert(baseAddresses IsNot Nothing)
            Debug.Assert(baseAddresses.Length > 0)

            m_SingletonInstance = singletonInstance

            For index As Integer = 0 To baseAddresses.Length - 1
                If baseAddresses(index).EndsWith("/") = False Then
                    baseAddresses(index) &= "/"
                End If
            Next index

            m_BaseAddresses = baseAddresses

            'Try to guess a certificate 
            ConfigureAnonymousMessageSecurity(ServiceBusHelper.ExtractSolution(New Uri(baseAddresses(0))))
        End Sub
        Public Sub New(ByVal serviceType As Type, ByVal baseAddress As String)
            Me.New(serviceType, New String() {baseAddress})
        End Sub
        Public Sub New(ByVal serviceType As Type, ByVal baseAddresses() As String)
            Debug.Assert(baseAddresses IsNot Nothing)
            Debug.Assert(baseAddresses.Length > 0)

            m_SericeType = serviceType

            For index As Integer = 0 To baseAddresses.Length - 1
                If baseAddresses(index).EndsWith("/") = False Then
                    baseAddresses(index) &= "/"
                End If
            Next index
            m_BaseAddresses = baseAddresses

            'Try to guess a certificate 
            ConfigureAnonymousMessageSecurity(ServiceBusHelper.ExtractSolution(New Uri(baseAddresses(0))))
        End Sub


        Public Sub SetServiceBusCertificate(ByVal findValue As Object, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType)
            m_ServiceBusCertFindValue = findValue
            m_ServiceBusCertLocation = location
            m_ServiceBusCertStoreName = storeName
            m_ServiceBusCertFindType = findType
        End Sub
        Public Sub SetServiceBusCertificate(ByVal subjectName As String)
            SetServiceBusCertificate(subjectName, StoreLocation.LocalMachine, StoreName.My, X509FindType.FindBySubjectName)
        End Sub
        Public Sub SetServiceBusPassword(ByVal password As String)
            m_ServiceBusPassword = password
        End Sub

        Public Sub ConfigureAnonymousMessageSecurity(ByVal serviceCert As String)
            ConfigureAnonymousMessageSecurity(serviceCert, StoreLocation.LocalMachine, StoreName.My)
        End Sub
        Public Sub ConfigureAnonymousMessageSecurity(ByVal serviceCert As String, ByVal location As StoreLocation, ByVal storeName As StoreName)
            ConfigureAnonymousMessageSecurity(location, storeName, X509FindType.FindBySubjectName, serviceCert)
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub ConfigureAnonymousMessageSecurity(ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType, ByVal findValue As Object)
            m_ServiceCertLocation = location
            m_ServiceCertStoreName = storeName
            m_ServiceCertFindType = findType
            m_ServiceCertFindValue = findValue
            m_Anonymous = True
        End Sub

        Public Sub ConfigureMessageSecurity(ByVal serviceCert As String)
            ConfigureMessageSecurity(serviceCert, StoreLocation.LocalMachine, StoreName.My, True, Nothing)
        End Sub
        Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal applicationName As String)
            ConfigureMessageSecurity(serviceCert, StoreLocation.LocalMachine, StoreName.My, True, applicationName)
        End Sub
        Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal useProviders As Boolean, ByVal applicationName As String)
            ConfigureMessageSecurity(serviceCert, StoreLocation.LocalMachine, StoreName.My, useProviders, applicationName)
        End Sub
        Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal useProviders As Boolean, ByVal applicationName As String)
            ConfigureMessageSecurity(location, storeName, X509FindType.FindBySubjectName, serviceCert, useProviders, applicationName)
        End Sub
        <MethodImpl(MethodImplOptions.Synchronized)> _
        Private Sub ConfigureMessageSecurity(ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType, ByVal findValue As Object, ByVal useProviders As Boolean, ByVal applicationName As String)
            m_ServiceCertLocation = location
            m_ServiceCertStoreName = storeName
            m_ServiceCertFindType = findType
            m_ServiceCertFindValue = findValue
            m_UseProviders = useProviders
            m_ApplicationName = applicationName
            m_Anonymous = False
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub SetBinding(ByVal binding As NetEventRelayBinding)
            m_Binding = binding
        End Sub

        Public Sub SetBinding(ByVal bindingConfigName As String)
            SetBinding(New NetEventRelayBinding(bindingConfigName))
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Subscribe()
            Dim serviceType As Type
            If m_SericeType IsNot Nothing Then
                serviceType = m_SericeType
            Else
                serviceType = m_SingletonInstance.GetType()
            End If
            Dim interfaces() As Type = serviceType.GetInterfaces()
            For Each interfaceType As Type In interfaces
                If interfaceType.GetCustomAttributes(GetType(ServiceContractAttribute), False).Length = 1 Then
                    Subscribe(interfaceType)
                End If
            Next interfaceType
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Subscribe(ByVal contractType As Type)
            Dim operations() As String = GetOperations(contractType)

            For Each operationName As String In operations
                Subscribe(contractType, operationName)
            Next operationName
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Subscribe(ByVal contractType As Type, ByVal operation As String)
            Debug.Assert(String.IsNullOrEmpty(operation) = False)

            If m_Hosts.ContainsKey(contractType) = False Then
                m_Hosts(contractType) = New Dictionary(Of String, ServiceBusHost)()

                Dim operations() As String = GetOperations(contractType)
                For Each operationName As String In operations
                    m_Hosts(contractType)(operationName) = Nothing
                Next operationName
            End If
            If m_Hosts(contractType)(operation) Is Nothing Then
                Dim baseAddressesList As List(Of Uri) = New List(Of Uri)()

                For Each address As String In m_BaseAddresses
                    Debug.Assert(address.Contains(ServiceBusEnvironment.DefaultRelayHostName))

                    baseAddressesList.Add(New Uri(address & contractType.ToString))
                Next address

                If m_SericeType IsNot Nothing Then
                    m_Hosts(contractType)(operation) = New ServiceBusHost(m_SericeType, baseAddressesList.ToArray())
                Else
                    m_Hosts(contractType)(operation) = New ServiceBusHost(m_SingletonInstance, baseAddressesList.ToArray())
                End If

                Dim binding As NetEventRelayBinding = If((Not m_Binding Is Nothing), m_Binding, New NetEventRelayBinding())

                m_Hosts(contractType)(operation).AddServiceEndpoint(contractType, binding, operation)

                'Configure service bus credentials for the host
                If m_ServiceBusPassword IsNot Nothing Then
                    m_Hosts(contractType)(operation).SetServiceBusPassword(m_ServiceBusPassword)
                End If
                If m_ServiceBusCertFindValue IsNot Nothing Then
                    m_Hosts(contractType)(operation).SetServiceBusCertificate(m_ServiceBusCertFindValue, m_ServiceBusCertLocation, m_ServiceBusCertStoreName, m_ServiceBusCertFindType)
                End If

                'Configure message security
                If m_Anonymous Then
                    m_Hosts(contractType)(operation).ConfigureAnonymousMessageSecurity(m_ServiceCertLocation, m_ServiceCertStoreName, m_ServiceCertFindType, m_ServiceCertFindValue)
                Else
                    m_Hosts(contractType)(operation).ConfigureMessageSecurity(m_ServiceCertLocation, m_ServiceCertStoreName, m_ServiceCertFindType, m_ServiceCertFindValue, m_UseProviders, m_ApplicationName)
                End If

                m_Hosts(contractType)(operation).Open()
            End If
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Unsubscribe()
            For Each contractType As Type In m_Hosts.Keys
                Unsubscribe(contractType)
            Next contractType
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Unsubscribe(ByVal contractType As Type)
            Dim operations() As String = GetOperations(contractType)

            For Each operationName As String In operations
                Unsubscribe(contractType, operationName)
            Next operationName
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Unsubscribe(ByVal contractType As Type, ByVal operation As String)
            Debug.Assert(String.IsNullOrEmpty(operation) = False)

            If m_Hosts.ContainsKey(contractType) = False Then
                Return
            End If
            If m_Hosts(contractType)(operation) IsNot Nothing Then
                m_Hosts(contractType)(operation).Close()
                m_Hosts(contractType)(operation) = Nothing
            End If
        End Sub

        Private Shared Function GetOperations(ByVal contract As Type) As String()
            Dim methods() As MethodInfo = contract.GetMethods(BindingFlags.Public Or BindingFlags.FlattenHierarchy Or BindingFlags.Instance)
            Dim operations As List(Of String) = New List(Of String)(methods.Length)

            Dim closure As New GetOperationsClosure
            closure.operations = operations
            Dim add As Action(Of MethodInfo) = AddressOf closure.Action

            methods.ForEach(add)
            Return operations.ToArray()
        End Function

        Private Class GetOperationsClosure
            Public operations As List(Of String)

            Public Sub Action(ByVal method As MethodInfo)
                Debug.Assert((Not operations.Contains(method.Name)))
                operations.Add(method.Name)
            End Sub
        End Class

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Abort()
            For Each contractType As Type In m_Hosts.Keys
                For Each host As ServiceHost In m_Hosts(contractType).Values
                    If host IsNot Nothing Then
                        host.Abort()
                    End If
                Next host
            Next contractType
        End Sub
    End Class
End Namespace
